#!/usr/bin/perl
#
# rccmd_conf.pl
# rccmd_adv configuration editor
#
#-----------  Change History  ---------------
use constant VERSION	=> '1.0.0';
#2006/11/02
#
#

#-----------  IO Setting  ---------------
$|=1;

#-----------  Include module  ---------------
use strict;
use vars qw();

#-----------  Constant definition  ---------------
use constant MAX_COL		=> 80;
use constant MAX_ROW		=> 25;
use constant HR			=> '#' x MAX_COL. "\n";

use constant STR_NOFILE		=> 0;
use constant STR_SAVE		=> 1;
use constant STR_QUIT		=> 2;
use constant STR_NUM		=> 3;
use constant STR_BACK		=> 4;
use constant STR_SET		=> 5;

#-----------  Global variable definition  ---------------
my $gVer = VERSION;
my $gEncode = 'e';
my $gConffile = 'rccmd_adv.conf';
my %CONF;
my %DESC;
my @strings = (
	" ե뤬Ĥޤ\n",
	"    's' ¸ƽλ",
	"    'q' ¸˽λ",
	"    ԽꥭֹϤƲ>",
	"β̤ˤϡ'b'ϤƤ\n",
	"ͤϤƲ",
);
my @keys = (
	'',
	'UPSMAN_PC',
	'CHECK_RATE',
	'SHUTDOWN',
	'SHUTDOWN_DELAY',
	'SHUTDOWN_START',
	'RCCMD_ECHO_PC',
	'RCCMD_POPUP',
	'RCCMD_POPUPTIME',
	'SHUTDOWN_MODE',
	'UPSTRAP',
	'COMMUNITY',
	'TRAP_POPUP',
	'TRAP_POPUPTIME',
	'TRAP_ECHO_PC',
	'TRAP',
);

#-----------  Hook of error  ---------------
local $SIG{__DIE__} = \&destroy;
local $SIG{INT} = \&destroy;


&Init();
&Main();
exit;

sub destroy
{
	my $msg = shift;
	print "$msg";
	exit;
}

sub Usage
{
	print "rccmd_conf version: $gVer\n";
#	      0         1         2         3         4         5         6         7         8
#	       12345678901234567890123456789012345678901234567890123456789012345678901234567890
	print "Usage: rccmd_conf.pl [-(utf|euc)] [-h] conf_file\n";
	print "   conf_file    Specify the configuration file for rccmd_adv.\n";
	print "                (Default is rccmd_adv.conf in current directory.)\n";
	print "\n";
	print "Options:\n";
	print "   -utf         The configuration file is encoded by UTF-8.\n";
	print "   -euc         The configuration file is encoded by EUC-JP.\n";
	print "                (Default encoded by EUC-JP)\n";
	print "   -h           Show this message. (help message and usage.)\n";
	print "\n";
	exit -1;
}

sub Init
{
	my $flg;
	foreach (@ARGV)
	{
		if ($_ =~ /^-/)
		{
			$flg = substr($_, 1, 1);
			&Usage if($flg eq '?' || $flg eq 'h');

			# Τʤѥ᡼
			$gEncode = $flg if($flg eq 'u');
			$gEncode = $flg if($flg eq 'e');
			next;
		}
		$gConffile = $_;
	}

	if ($gEncode eq 'u'){
		map { $_ = `echo -n \"$_\" | ./nkf -w` } @strings;
	}

	die $gConffile.$strings[STR_NOFILE] unless( -e $gConffile);

	# Configurationɤ߹
	&conf_read();
	&read_desc();

	return;
}

sub Main
{
	my ($key, $val);
	while (1)
	{
		&PrintHeader();
		my ($key, $val);
		my $num = 1;
		foreach (@keys){
			next if($_ eq "");

			$key = $_;
			$val = $CONF{$key};
			if ($key eq 'TRAP')
			{
				printf("    %2d. %15s = ", $num, $key);
				print "1  2  3  4  5  6  7  8  9 10 12 13 14 15 16 17\n";
				print "                          ";
				for (my $i = 1; $i <= 17; $i++)
				{
					next      if($i == 11);
					if ($CONF{"TRAP_$i"} =~ /^D/i){
						print "D  ";
					}else{
						print "E  ";
					}
				}
				print "\n";
			}
			else
			{
				substr($val, 50) = "...\0" if(length($val) > 50);
				printf("    %2d. %15s = %s\n", $num, $key, $val);
			}
			$num++
		}
		print "\n";
		print $strings[STR_SAVE], "\n";
		print $strings[STR_QUIT], "\n";
		print $strings[STR_NUM];

		my $in = <STDIN>;
		chomp($in);

		if ($in eq 's'){
			print "saving.\n";
			&conf_write();
			last;
		}elsif ($in eq 'q'){
			print "quit.\n";
			last;
		}

		$key = $keys[$in];
		$val = $CONF{$key};
		next unless($key);
		if ($key eq "TRAP")
		{
			while (1)
			{
				&PrintHeader();
				for (my $i = 1; $i <= 17; $i++)
				{
					next if($i == 11);
					printf("    %2d. %8s = %s\n", $i, "TRAP_$i", $CONF{"TRAP_$i"});
				}
				print "\n";
				print $strings[STR_BACK], "\n";
				print $strings[STR_NUM];
				$in = <STDIN>;
				chomp($in);
				last if($in eq 'b');

				$key = "TRAP_$in";
				$val = $CONF{$key};
				last if($in >= 1 && $in <= 17 && $in != 11);
			}
			next if($in eq 'b');
		}

		my $desc = $DESC{$key};
		$desc .= "\n".$DESC{'TRAP'} if($key =~ /^TRAP_[0-9]+/);

		&PrintHeader();
		print "$key = $val\n";
		print "\n";
		print '#' x MAX_COL, "\n";
		print "\n";
		print $desc, "\n";
		print '#' x MAX_COL, "\n";
		print "\n";
		print $strings[STR_BACK], "\n";
		print $strings[STR_SET], "\n";
		print "$key = ";
		$in = <STDIN>;
		chomp($in);
		$CONF{$key} = $in unless($in eq 'b');
	}

	return;
}

sub conf_read
{
	open (IN, "$gConffile"); my @confs= <IN>; close (IN);

	foreach $_ (@confs){
		$_ =~ s/[\r\n]//;
		$_ =~ s/\s//g;
		next if($_ eq "");
		next if($_ =~ /^#/);
		my ($key, $set) = split(/=/, $_);
#		$set =~ tr/[A-Z]/[a-z]/;       # ʸϱѾʸ
		$key =~ tr/[a-z]/[A=Z]/;       # Keyʸϱʸ
		$CONF{$key} = $set;
	}

	# Parameter setting
	$CONF{'CHECK_RATE'} = 10;
	&SetRange(60, 999, \$CONF{'SHUTDOWN_DELAY'});
	&SetRange(10, 999, \$CONF{'RCCMD_POPUPTIME'});
	&SetRange(10, 999, \$CONF{'TRAP_POPUPTIME'});
	my @tmp = split(/,/, $CONF{'RCCMD_ECHO_PC'});
	if (@tmp > 20){
		splice(@tmp, 20);	# ڤ
		$CONF{'RCCMD_ECHO_PC'} = join(', ', @tmp);
	}
	@tmp = split(/,/, $CONF{'TRAP_ECHO_PC'});
	if (@tmp > 20){
		splice(@tmp, 20);	# ڤ
		$CONF{'TRAP_ECHO_PC'} = join(', ', @tmp);
	}

	return;
}

sub conf_write
{
	open (IN, "$gConffile"); my @dat = <IN>; close(IN);
	foreach (@dat)
	{
		chomp($_);
		next if($_ eq "");
		next if($_ =~ /^#/);

		my ($key, undef) = split(/=/, $_);
		$key =~ tr/[a-z]/[A=Z]/;       # Keyʸϱʸ
		$key =~ s/\s//g;
		$_ = join(" = ", $key, $CONF{$key});
	}
	my $config = join("\n", @dat);
	open (OUT, ">$gConffile"); print OUT $config; close(OUT);
}

sub read_desc
{
	my $name = 'description';
	if ($gEncode eq 'u'){
		`./nkf -w -O $name $name.utf`;
		$name .= '.utf';
	}
	open(IN, $name); my @dat = <IN>; close(IN);
	unlink($name) if($gEncode eq 'u');

	foreach (@dat){
		next if($_ =~ /^#/);
		my ($key, $val) = split(/=/, $_);
		$key =~ s/^ *//;
		$key =~ s/ *$//;
		$val =~ s/^ *//;
		$val =~ s/ *$//;
		$val =~ s/\\n/\n/g;
		$DESC{$key} = $val;
	}
}

sub SetRange()
{
	my ($min, $max, $var) = @_;

	# default set
	$$var = $$var || $min;

	# range set
	$$var = $min if($$var < $min);
	$$var = $max if($$var > $max);
}

sub PrintHeader
{
	system("clear");
	print "RCCMD ADVANCED Configuration.\n";
	print HR;
	print "\n";
}
